import 'package:flutter/material.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:mobiles_labs_5th_semester/data/repositories/mock_repository.dart';
import 'package:mobiles_labs_5th_semester/domain/models/game.dart';
import 'package:mobiles_labs_5th_semester/main.dart';
import 'package:mobiles_labs_5th_semester/presentation/details_page/details_page.dart';
import 'package:mobiles_labs_5th_semester/presentation/home_page/bloc/bloc.dart';
import 'package:mobiles_labs_5th_semester/presentation/home_page/bloc/events.dart';
import 'package:mobiles_labs_5th_semester/presentation/home_page/bloc/state.dart';

import '../../components/utils/debounce.dart';
import '../../data/repositories/games_repository.dart';
import 'bloc/state.dart';

part 'gameCard.dart';

class HomePage extends StatefulWidget {
  const HomePage({super.key, required this.title});

  final String title;

  @override
  State<HomePage> createState() => _HomePageState();
}

class _HomePageState extends State<HomePage> {
  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        backgroundColor: Color.fromARGB(255, 46, 65, 80),
        appBar: AppBar(
          backgroundColor: Color.fromARGB(255, 56, 90, 128),
          title: Text(
            widget.title,
            style: const TextStyle(
                color: Colors.white, fontWeight: FontWeight.bold),
          ),
        ),
        body: const Body());
  }
}

//!!!скорее всего надо будет заменить на stateful
class Body extends StatefulWidget {
  const Body({super.key});

  @override
  State<Body> createState() => _BodyState();
}

class _BodyState extends State<Body> {
  late Future<List<GameData>?> data;
  final searchController = TextEditingController();
  final scrollController = ScrollController();

  // final repo = GamesRepository();

  @override
  void initState() {
    //добавление кастомного события
    WidgetsBinding.instance.addPostFrameCallback((_) {
      context.read<HomeBloc>().add(const HomeLoadDataEvent());
    });

    scrollController.addListener(_onNextPageListener);
    super.initState();
  }

  void _onNextPageListener() {
    if (scrollController.offset >= scrollController.position.maxScrollExtent) {
      final bloc = context.read<HomeBloc>();
      if (!bloc.state.isPaginationLoading) {
        bloc.add(HomeLoadDataEvent(
          search: searchController.text,
          nextPage: bloc.state.data?.nextPage,
        ));
      }
    }
  }

  @override
  void dispose() {
    searchController.dispose();
    scrollController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Column(
      children: [
        Padding(
          padding: const EdgeInsets.only(top: 12, left: 12, right: 12),
          child: CupertinoSearchTextField(
            backgroundColor: Colors.white,
            style: TextStyle(color: Color.fromARGB(255, 46, 65, 80)),
            // borderRadius: const BorderRadiusTween(2.0),
            controller: searchController,
            onChanged: (search) {
              Debounce.run(() => context
                  .read<HomeBloc>()
                  .add(HomeLoadDataEvent(search: search)));
            },
          ),
        ),
        BlocBuilder<HomeBloc, HomeState>(
          builder: (context, state) => state.error != null
              ? Text(
                  state.error ?? '',
                  style: Theme.of(context)
                      .textTheme
                      .headlineSmall
                      ?.copyWith(color: Colors.red),
                )
              : state.isLoading
                  ? const CircularProgressIndicator()
                  : Expanded(
                      child: RefreshIndicator(
                        onRefresh: _onRefresh,
                        child: ListView.builder(
                          controller: scrollController,
                          itemCount: state.data?.data?.length ?? 0,
                          itemBuilder: (context, index) {
                            final data = state.data?.data?[index];
                            return data != null
                                ? _GameCard.fromData(
                                    data,
                                    onLike: (title, isLiked) =>
                                        _showSnackBar(context, title, isLiked),
                                    onTap: () => _navToDetails(context, data),
                                  )
                                : const SizedBox.shrink();
                          },
                        ),
                      ),
                    ),
        ),
        //значок загрузки при пагинации
        BlocBuilder<HomeBloc, HomeState>(
          builder: (context, state) => state.isPaginationLoading
              ? const CircularProgressIndicator()
              : const SizedBox.shrink(),
        )
      ],
    );
  }

  Future<void> _onRefresh() {
    context
        .read<HomeBloc>()
        .add(HomeLoadDataEvent(search: searchController.text));
    return Future.value(null);
  }

  void _showSnackBar(BuildContext context, String title, bool isLiked) {
    WidgetsBinding.instance.addPostFrameCallback((_) {
      ScaffoldMessenger.of(context).showSnackBar(SnackBar(
        content: Text(
          isLiked
              ? 'Вы поставили лайк игре "$title"'
              : 'Вы убрали лайк у игры "$title"',
          style: Theme.of(context)
              .textTheme
              .bodyLarge
              ?.copyWith(color: Colors.pink, fontWeight: FontWeight.bold),
          textAlign: TextAlign.center,
        ),
        backgroundColor: Colors.white,
        duration: const Duration(seconds: 1),
      ));
    });
  }

  void _navToDetails(BuildContext context, GameData data) {
    Navigator.push(
      context,
      CupertinoPageRoute(builder: (context) => DetailsPage(data.id ?? 0)),
    );
  }
}
